﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace txt2exe
{
    public partial class frmmain : Form
    {
        public frmmain()
        {
            InitializeComponent();
        }

        private int lo(int number)
        {
            return number & 0xFF;
        }

        private void ShowError(int id, string sMsg)
        {
            string[] message = new string[3];

            string msg = "Error: " + id.ToString();
            msg += Environment.NewLine;
            //Set error messages.
            message[0] = "Input read error.";
            message[1] = "Output write error.";
            message[2] = sMsg;

            //Append error message.
            msg += message[id];
            //Display error message.
            MessageBox.Show(msg,
                "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private long hi(int number)
        {
            return number >> 8;
        }

        private int txt2Exe(string Input, string Output)
        {
            FileInfo InputFile = null;
            FileInfo OutputFile = null;

            byte[] header = new byte[15];
            byte[] fBytes;
            int fSize = 0;

            //Set input and output file.
            InputFile = new FileInfo(Input);
            OutputFile = new FileInfo(Output);

            //Exe loader machine code.
            header[0] = 0xBE;
            header[1] = 0x0F;
            header[2] = 0x01;
            header[3] = 0xB9;
            header[4] = 0x00;
            header[5] = 0x00;
            header[6] = 0xFC;
            header[7] = 0xAC;
            header[8] = 0xCD;
            header[9] = 0x29;
            header[10] = 0x49;
            header[11] = 0x75;
            header[12] = 0xFA;
            header[13] = 0xCD;
            header[14] = 0x20;

            try
            {
                using (BinaryReader br = new BinaryReader(File.OpenRead(InputFile.FullName)))
                {
                    //Get filezie.
                    fSize = (int)br.BaseStream.Length;
                    //Read bytes from the input file.
                    fBytes = br.ReadBytes(fSize);
                    //Close input file.
                    br.Close();
                }
            }
            catch
            {
                //Exit if we have any errors.
                return 0;
            }

            try
            {
                //Edit bytes 4 and 5
                //Set byte 4 to low byte of filesize.
                header[4] = (byte)lo(fSize);
                //Set byte 5 to high byte of filezize.
                header[5] = (byte)hi(fSize);

                //Open the output file for writeing.
                using (BinaryWriter bw = new BinaryWriter(File.OpenWrite(OutputFile.FullName)))
                {
                    //Write the exe header to output file.
                    bw.Write(header, 0, 15);
                    //Write the input file bytes to output file.
                    bw.BaseStream.Write(fBytes, 0, fSize);
                    //Close output file.
                    bw.Close();
                }
            }
            catch
            {
                //Clear up.
                Array.Clear(header, 0, header.Length);
                Array.Clear(fBytes, 0, fBytes.Length);
                //Exit if we have errors.
                return 1;
            }
            //Clear up.
            Array.Clear(header, 0, header.Length);
            Array.Clear(fBytes, 0, fBytes.Length);
            //Return flag.
            return 2;
        }

        private void cmdOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog od = new OpenFileDialog();
            int sPos = -1;

            od.Title = "Choose File";
            od.Filter = "Text Files(*.txt)|*.txt";

            if (od.ShowDialog() == DialogResult.OK)
            {
                txtInput.Text = od.FileName;
                //Set a temp name for the output filename.
                sPos = od.FileName.LastIndexOf('.');
                if (sPos != -1)
                {
                    //Make output filename.
                    txtOutput.Text = od.FileName.Substring(0, sPos + 1) + "exe";
                }
            }
        }

        private void cmdConvert_Click(object sender, EventArgs e)
        {
            FileInfo f1 = null;
            FileInfo f2 = null;
            int RetVal = 0;
            try
            {
                //Get input and output filenames.
                f1 = new FileInfo(txtInput.Text);
                f2 = new FileInfo(txtOutput.Text);
                //Convert text to exe.
                RetVal = txt2Exe(f1.FullName, f2.FullName);
                //Check return value.
                if (RetVal != 2)
                {
                    //Display error.
                    ShowError(RetVal,string.Empty);
                }
            }
            catch (Exception err)
            {
                ShowError(2, err.Message);
            }
        }

        private void cmdSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog sa = new SaveFileDialog();

            sa.Title = "Choose File";
            sa.Filter = "Appliaction Files(*.exe)|*.exe";
            if (sa.ShowDialog() == DialogResult.OK)
            {
                txtOutput.Text = sa.FileName;
            }
        }

        private void cmdExit_Click(object sender, EventArgs e)
        {
            //Close program.
            Close();
        }
    }
}
